# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from PIL import Image
# from shutil import rmtree

from towhee import pipeline
from towhee.errors import OpFailedError
from towhee.tests import CACHE_PATH
from towhee.hub.file_manager import FileManagerConfig, FileManager


class TestPipeline(unittest.TestCase):
    """
    Tests `pipeline` functionality.
    """
    @classmethod
    def setUpClass(cls):
        new_cache = (CACHE_PATH/'test_cache')
        pipeline_cache = (CACHE_PATH/'test_util')
        operator_cache = (CACHE_PATH/'mock_operators')
        fmc = FileManagerConfig()
        fmc.update_default_cache(new_cache)
        pipelines = list(pipeline_cache.rglob('*.yaml'))
        operators = [f for f in operator_cache.iterdir() if f.is_dir()]
        fmc.cache_local_pipeline(pipelines)
        fmc.cache_local_operator(operators)
        FileManager(fmc)

    def test_empty_input(self):
        p = pipeline('local/simple_pipeline')
        with self.assertRaises(RuntimeError):
            p()

    def test_simple_pipeline(self):
        p = pipeline('local/simple_pipeline')
        res = p(0)
        self.assertEqual(res[0][0], 3)

    def test_flatmap_pipeline(self):
        p = pipeline('local/flatmap_pipeline')
        res = p(10)
        self.assertEqual(len(res), 5)

    def test_filter_pipeline(self):
        p = pipeline('local/filter_pipeline')
        res = p([1, 0, 2, 0, 3, 4])
        num = 1
        for item in res:
            self.assertEqual(item[0], num)
            num += 1

    def test_embedding_pipeline(self):
        p = pipeline('local/resnet50_embedding')
        img_path = CACHE_PATH / 'data' / 'dataset' / 'kaggle_dataset_small' / \
            'train' / '0021f9ceb3235effd7fcde7f7538ed62.jpg'
        img = Image.open(str(img_path))
        res = p(img)
        self.assertEqual(res[0][0].size, 1000)

    def test_error_input(self):
        p = pipeline('local/simple_pipeline')
        with self.assertRaises(OpFailedError):
            p('xx')

    def test_test_concat(self):
        p = pipeline('local/test_concat')
        res = p(0, 0, 0)
        self.assertEqual(res, [(1,0,2,0,3,0)])

    # def test_simple_pipeline_multirow(self):
    #     #pylint: disable=protected-access
    #     p = pipeline('test_util/simple_pipeline', cache=str(CACHE_PATH))
    #     p._pipeline.parallelism = 2
    #     res = p(list(range(1000)))
    #     for n in range(1000):
    #         self.assertEqual(res[n], n+3)


# class TestPipelineCache(unittest.TestCase):
#     def test_pipeline_cache(self):
#         self.assertEqual(_get_pipeline_cache(
#             None), Path.home() / '.towhee/pipelines')

#         os.environ[_PIPELINE_CACHE_ENV] = '/opt/.pipeline'
#         self.assertEqual(_get_pipeline_cache(
#             None), Path('/opt/.pipeline'))

if __name__ == '__main__':
    unittest.main()



# coding=utf-8
# Copyright 2020-present the HuggingFace Inc. team and 2021 Zilliz.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import random
import math

import numpy
import numpy as np


class RandomErasing:
    """
    Random erasing the an rectangle region in Image.
    Class that performs Random Erasing in Random Erasing Data Augmentation by Zhong et al.

    Args:
        sl (`float`): min erasing area region
        sh (`float`): max erasing area region
        r1 (`float`): min aspect ratio range of earsing region
        p (`float`): probability of performing random erasing
    """

    def __init__(self, p: float = 0.5, sl: float = 0.02, sh: float = 0.4, r1: float = 0.3):
        self.p = p
        self.s = (sl, sh)
        self.r = (r1, 1 / r1)

    def __call__(self, img: numpy.ndarray):
        """
        perform random erasing
        Args:
            img (`numpy.ndarray`): opencv numpy array in form of [w, h, c] range
                 from [0, 255]

        Returns:
            (`numpy.ndarray`)
                erased img.
        """

        assert len(img.shape) == 3, 'image should be a 3 dimension numpy array'

        if random.random() > self.p:
            return img

        else:
            while True:
                Se = random.uniform(*self.s) * img.shape[0] * img.shape[1]
                re = random.uniform(*self.r)

                He = int(round(math.sqrt(Se * re)))
                We = int(round(math.sqrt(Se / re)))

                xe = random.randint(0, img.shape[1])
                ye = random.randint(0, img.shape[0])

                if xe + We <= img.shape[1] and ye + He <= img.shape[0]:
                    img[ye: ye + He, xe: xe + We, :] = np.random.randint(low=0, high=255, size=(He, We, img.shape[2]))

                    return img



# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import NamedTuple
from towhee.operator import PyOperator, SharedType


class AddOperator(PyOperator):
    """
    Stateful operator
    """
    def __init__(self, factor: int) -> None:
        super().__init__()
        self._factor = factor

    def __call__(self, num: int) -> NamedTuple("Outputs", [("sum", int)]):
        Outputs = NamedTuple("Outputs", [("sum", int)])
        return Outputs(self._factor + num)

    @property
    def shared_type(self):
        return SharedType.Shareable



# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
import torch

from towhee.models.layers.aspp import ASPP


class TestOperator(unittest.TestCase):
    """
    A CNN model which adds spp layer so that we can input multi-size tensor
    """

    def setUp(self):
        self.input = torch.rand((2, 512, 13, 13))

    def test_spp(self):
        aspp_module = ASPP()
        self.assertEqual(256, aspp_module(self.input).shape[1])



# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Pytorch impletation of 'An Image is Worth 16x16 Words: Transformers for Image Recognition at Scale'.
Jax version: https://github.com/google-research/vision_transformer.
"""

import torch
from torch import nn

from towhee.trainer.models.vit.transformer import Transformer
from towhee.trainer.models.vit.vit_training_utils import load_pretrained_weights, as_tuple
from towhee.trainer.models.vit.vit_pretrained import PRETRAINED_MODELS


class PositionalEmbedding1D(nn.Module):
    """
    Adds (optionally learned) positional embeddings to the inputs.
    """

    def __init__(self, seq_len, dim):
        super().__init__()
        self.pos_embedding = nn.Parameter(torch.zeros(1, seq_len, dim))

    def forward(self, x):
        """
        Input has shape (batch_size, seq_len, emb_dim)
        """
        return x + self.pos_embedding


class ViT(nn.Module):
    """
    Args:
        name(str, optional):
            Model name.
        weights_path(str):
            Local path to weights file.
        pretrained(bool):
            Load pretrained weights.
        patches(int):
            Number of image patches.
        dim(int):
            Number of self dimention.
        ff_dim(int):
            Number of layer size in feedforward network.
        num_heads(int):
            Number of heads.
        num_layers(int):
            Number of layers.
        attention_dropout_rate(float):
            Attention dropout rate.
        dropout_rate(float):
            Dropout rate.
        representation_size(int, optional):
            Enable and set representation layer to this value if set.
        classifier(str):
            classifier type.
        positional_embedding(str):
            positional embedding.
        in_channels(int):
            Number of input channels.
        image_size(int, optional):
            image size.
        num_classes(int, optional):
            Number of classes for classification head.
        resize_positional_embedding(bool,optional):
            If resize_positional_embedding is true.
        to_vec(bool, optional):
            If the embedding vector is predicted instead.
    """

    def __init__(
        self,
        name=None,
        weights_path=None,
        pretrained=False,
        patches=16,
        dim=768,
        ff_dim=3072,
        num_heads=12,
        num_layers=12,
        attention_dropout_rate=0.0,
        dropout_rate=0.1,
        representation_size=None,
        load_repr_layer=False,
        classifier='token',
        positional_embedding='1d',
        in_channels=3,
        image_size=None,
        num_classes=None,
        resize_positional_embedding=True,
        to_vec=False
    ):
        super().__init__()
        print(f'attention_dropout_rate is {attention_dropout_rate}')
        # Configuration
        if name is None:
            check_msg = 'must specify name of pretrained model'
            assert not pretrained, check_msg
            assert not resize_positional_embedding, check_msg
            if num_classes is None:
                num_classes = 1000
            if image_size is None:
                image_size = 384
        else:  # load pretrained model
            assert name in PRETRAINED_MODELS, \
                'name should be in: ' + ', '.join(PRETRAINED_MODELS.keys())
            config = PRETRAINED_MODELS[name]['config']
            patches = config['patches']
            dim = config['dim']
            ff_dim = config['ff_dim']
            num_heads = config['num_heads']
            num_layers = config['num_layers']
            attention_dropout_rate = config['attention_dropout_rate']
            dropout_rate = config['dropout_rate']
            representation_size = config['representation_size']
            classifier = config['classifier']
            if image_size is None:
                image_size = PRETRAINED_MODELS[name]['image_size']
            if num_classes is None:
                num_classes = PRETRAINED_MODELS[name]['num_classes']
        self.image_size = image_size

        self.to_vec = to_vec

        # Image and patch sizes
        h, w = as_tuple(image_size)  # image sizes
        fh, fw = as_tuple(patches)  # patch sizes
        gh, gw = h // fh, w // fw  # number of patches
        seq_len = gh * gw

        # Patch embedding
        self.patch_embedding = nn.Conv2d(in_channels, dim, kernel_size=(fh, fw), stride=(fh, fw))

        # Class token
        if classifier == 'token':
            self.class_token = nn.Parameter(torch.zeros(1, 1, dim))
            seq_len += 1

        # Positional embedding
        if positional_embedding.lower() == '1d':
            self.positional_embedding = PositionalEmbedding1D(seq_len, dim)
        else:
            raise NotImplementedError()

        # Transformer
        self.transformer = Transformer(num_layers=num_layers, dim=dim, num_heads=num_heads,
                                       ff_dim=ff_dim, dropout=dropout_rate)

        # Representation layer
        if representation_size and load_repr_layer:
            self.pre_logits = nn.Linear(dim, representation_size)
            pre_logits_size = representation_size
        else:
            pre_logits_size = dim

        # Classifier head
        self.norm = nn.LayerNorm(pre_logits_size, eps=1e-6)
        self.fc = nn.Linear(pre_logits_size, num_classes)

        # Initialize weights
        self.init_weights()

        # Load pretrained model
        if pretrained:
            pretrained_num_channels = 3
            pretrained_num_classes = PRETRAINED_MODELS[name]['num_classes']
            pretrained_image_size = PRETRAINED_MODELS[name]['image_size']
            load_pretrained_weights(
                self, name,
                weights_path=weights_path,
                load_first_conv=(in_channels == pretrained_num_channels),
                load_fc=(num_classes == pretrained_num_classes),
                load_repr_layer=load_repr_layer,
                resize_positional_embedding=(image_size != pretrained_image_size),
            )

    @torch.no_grad()
    def init_weights(self):
        def _init(m):
            if isinstance(m, nn.Linear):
                nn.init.xavier_uniform_(m.weight)
                if hasattr(m, 'bias') and m.bias is not None:
                    nn.init.normal_(m.bias, std=1e-6)  # nn.init.constant(m.bias, 0)
        self.apply(_init)
        nn.init.constant_(self.fc.weight, 0)
        nn.init.constant_(self.fc.bias, 0)
        nn.init.normal_(self.positional_embedding.pos_embedding, std=0.02)
        nn.init.constant_(self.class_token, 0)

    def forward(self, x):
        """
        Breaks image into patches, applies transformer, applies MLP head.

        Args:
            x (tensor):
                b,c,fh,fw
        """
        b, _, _, _ = x.shape
        x = self.patch_embedding(x)  # b,d,gh,gw
        x = x.flatten(2).transpose(1, 2)  # b,gh*gw,d
        if hasattr(self, 'class_token'):
            x = torch.cat((self.class_token.expand(b, -1, -1), x), dim=1)  # b,gh*gw+1,d
        if hasattr(self, 'positional_embedding'):
            x = self.positional_embedding(x)  # b,gh*gw+1,d
        x = self.transformer(x)  # b,gh*gw+1,d
        if hasattr(self, 'pre_logits'):
            x = self.pre_logits(x)
            x = torch.tanh(x)
        if hasattr(self, 'fc'):
            x = self.norm(x)[:, 0]  # b,d
            if self.to_vec:
                print('vit output embedding')
                return x
            x = self.fc(x)  # b,num_classes
        return x



import unittest
from pathlib import Path
from PIL import Image
# from shutil import rmtree

from towhee import pipeline
from towhee.hub.file_manager import FileManagerConfig, FileManager
from towhee.tests import CACHE_PATH

cache_path = Path(__file__).parent.parent.resolve()


@unittest.skip('Not pass')
class TestDownload(unittest.TestCase):
    """
    Simple hub download and run test.
    """
    @classmethod
    def setUpClass(cls):
        new_cache = (CACHE_PATH / 'test_cache')
        pipeline_cache = (CACHE_PATH / 'test_util')
        operator_cache = (CACHE_PATH / 'mock_operators')
        fmc = FileManagerConfig()
        fmc.update_default_cache(new_cache)
        pipelines = list(pipeline_cache.rglob('*.yaml'))
        operators = [f for f in operator_cache.iterdir() if f.is_dir()]
        fmc.cache_local_pipeline(pipelines)
        fmc.cache_local_operator(operators)
        FileManager(fmc)

    def test_pipeline(self):
        p = pipeline('towhee/ci_test')
        img = Image.open(CACHE_PATH / 'mock_pipelines/ci_test/towhee_logo.png')
        res = p(img)
        self.assertEqual(res[0][0].size, 1000)


if __name__ == '__main__':
    unittest.main()



# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from setuptools import setup, find_packages
from setuptools.command.install import install
import unittest
from pathlib import Path
from typing import List


def test_suite():
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover('towhee/tests', pattern='test_*.py')
    return test_suite


def create_towhee_cache(dst: str):
    if not Path(dst).is_dir():
        Path(dst).mkdir(parents=True)


def parse_requirements(file_name: str) -> List[str]:
    with open(file_name) as f:
        return [require.strip() for require in f if require.strip() and not require.startswith("#")]


setup(
    name="towhee",
    version="0.4.0rc1",
    description="",
    author="Towhee Team",
    author_email="towhee-team@zilliz.com",
    # use_scm_version={"local_scheme": "no-local-version"},
    setup_requires=['setuptools_scm'],
    url="https://github.com/towhee-io/towhee",
    test_suite="setup.test_suite",
    install_requires=parse_requirements('requirements.txt'),
    tests_require=parse_requirements('test_requirements.txt'),
    packages=find_packages(),
    package_data={'towhee.tests.test_util': ['*.yaml']},
    license="http://www.apache.org/licenses/LICENSE-2.0",
)



# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import torch
import unittest

from towhee.models.multiscale_vision_transformers.create_multiscale_vision_transformers \
    import create_multiscale_vision_transformers


class MultiscaleVisionTransformersTest(unittest.TestCase):
    def test_multiscale_vision_transformers(self):
        # Test MViT with 3D case.
        num_head = 100
        batch_size = 1
        fake_input = torch.rand(batch_size, 3, 4, 28, 28)
        model = create_multiscale_vision_transformers(
            spatial_size=28,
            temporal_size=4,
            patch_embed_dim=12,
            depth=1,
            head_num_classes=num_head,
            pool_kv_stride_adaptive=[1, 2, 2],
        )
        output = model(fake_input)
        gt_shape_tensor = torch.rand(batch_size, num_head)
        self.assertTrue(output.shape == gt_shape_tensor.shape)
        # Test MViT with 3D case with pool first.
        num_head = 100
        batch_size = 1
        fake_input = torch.rand(batch_size, 3, 4, 28, 28)
        model = create_multiscale_vision_transformers(
            spatial_size=28,
            temporal_size=4,
            patch_embed_dim=12,
            depth=1,
            head_num_classes=num_head,
            pool_first=True,
            pool_q_stride_size=[[0, 1, 2, 2]],
        )
        output = model(fake_input)
        gt_shape_tensor = torch.rand(batch_size, num_head)
        self.assertTrue(output.shape == gt_shape_tensor.shape)

        # Test MViT with 2D case for images.
        conv_patch_kernel = (7, 7)
        conv_patch_stride = (4, 4)
        conv_patch_padding = (3, 3)
        num_head = 100
        batch_size = 1
        fake_input = torch.rand(batch_size, 3, 28, 28)
        model = create_multiscale_vision_transformers(
            spatial_size=(28, 28),
            temporal_size=1,
            patch_embed_dim=12,
            depth=1,
            head_num_classes=num_head,
            use_2d_patch=True,
            conv_patch_embed_kernel=conv_patch_kernel,
            conv_patch_embed_stride=conv_patch_stride,
            conv_patch_embed_padding=conv_patch_padding,
        )
        output = model(fake_input)
        gt_shape_tensor = torch.rand(batch_size, num_head)
        self.assertTrue(output.shape == gt_shape_tensor.shape)

        # Test MViT without patch_embed.
        num_head = 100
        batch_size = 1
        fake_input = torch.rand(batch_size, 8, 12)
        model = create_multiscale_vision_transformers(
            spatial_size=(8, 1),
            temporal_size=1,
            patch_embed_dim=12,
            depth=1,
            enable_patch_embed=False,
            head_num_classes=num_head,
        )
        output = model(fake_input)
        gt_shape_tensor = torch.rand(batch_size, num_head)
        self.assertTrue(output.shape == gt_shape_tensor.shape)


if __name__ == '__main__':
    unittest.main()



# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import unittest

from towhee.models.utils.create_act import get_act_layer

class TestCreateAct(unittest.TestCase):
    """
    Test activation layer factory.
    """

    def test_createact(self):
        act_fn = get_act_layer('silu')
        self.assertTrue(act_fn is not None)

        act_fn = get_act_layer('swish')
        self.assertTrue(act_fn is not None)

        act_fn = get_act_layer('mish')
        self.assertTrue(act_fn is not None)

        act_fn = get_act_layer('relu')
        self.assertTrue(act_fn is not None)

        act_fn = get_act_layer('relu6')
        self.assertTrue(act_fn is not None)

        act_fn = get_act_layer('leaky_relu')
        self.assertTrue(act_fn is not None)

        act_fn = get_act_layer('prelu')
        self.assertTrue(act_fn is not None)

        act_fn = get_act_layer('celu')
        self.assertTrue(act_fn is not None)

        act_fn = get_act_layer('selu')
        self.assertTrue(act_fn is not None)

        act_fn = get_act_layer('gelu')
        self.assertTrue(act_fn is not None)

        act_fn = get_act_layer('sigmoid')
        self.assertTrue(act_fn is not None)

        act_fn = get_act_layer('tanh')
        self.assertTrue(act_fn is not None)

        act_fn = get_act_layer('hard_sigmoid')
        self.assertTrue(act_fn is not None)

        act_fn = get_act_layer('hard_swish')
        self.assertTrue(act_fn is not None)

        act_fn = get_act_layer('hard_mish')
        self.assertTrue(act_fn is not None)




# coding : UTF-8
import sys
sys.path.append("../../tests")
import time
import threading
from towhee import pipeline
from common import common_func as cf

embedding_size = 1000

class TestImageEmbeddingInvalid:
    """ Test case of invalid embedding interface """

    def test_embedding_no_parameter(self, pipeline_name):
        """
        target: test embedding for invalid scenario
        method:  embedding with no image, now issue 154
        expected: raise exception
        """
        embedding_pipeline = pipeline(pipeline_name)
        try:
            embedding = embedding_pipeline()
        except RuntimeError as e:
            assert "Input data is empty" in str(e)

        return True

    def test_embedding_wrong_parameter(self, pipeline_name):
        """
        target: test embedding for invalid scenario
        method:  embedding with no image, now issue 154
        expected: raise exception
        """
        embedding_pipeline = pipeline(pipeline_name)
        try:
            embedding = embedding_pipeline("")
        except RuntimeError as e:
            assert "'str' object has no attribute 'read'" in str(e)

        return True

    def test_embedding_not_exist_image(self, pipeline_name):
        """
        target: test embedding for invalid scenario
        method:  embedding with no image, now issue 154
        expected: raise exception
        """
        embedding_pipeline = pipeline(pipeline_name)
        not_exist_img = "towhee_no_exist.jpg"
        try:
            embedding = embedding_pipeline(not_exist_img)
        except RuntimeError as e:
            assert "No such file or directory" in str(e)

        return True

    def test_embedding_image_list(self, pipeline_name):
        """
        target: test embedding with image list
        method:  embedding with image list
        expected: raise exception
        """
        N = 3
        img_list = []
        img = cf.create_image()
        for i in range(N):
            img_list.append(img)
        embedding_pipeline = pipeline(pipeline_name)
        try:
            embedding = embedding_pipeline(img_list)
        except RuntimeError as e:
            assert "img should be PIL Image" in str(e)

        return True

class TestImageEmbeddingValid:
    """ Test case of valid embedding interface """

    def test_embedding_one_image(self, pipeline_name):
        """
        target: test embedding for normal case
        method:  embedding with one image
        expected: return embeddings
        """
        img = cf.create_image()
        embedding_pipeline = pipeline(pipeline_name)
        embedding = embedding_pipeline(img)
        assert embedding[0][0].size == embedding_size

        return True

    def test_embedding_same_images(self, pipeline_name):
        """
        target: test embedding for normal case
        method:  embedding with same images
        expected: return identical embeddings
        """
        img = cf.create_image()
        img_1 = img.copy()
        embedding_pipeline = pipeline(pipeline_name)
        embedding = embedding_pipeline(img)
        embedding1 = embedding_pipeline(img_1)
        assert embedding[0][0].size == embedding_size
        assert embedding1[0][0].size == embedding_size
        assert embedding[0][0].all() == embedding1[0][0].all()

        return True

    def test_embedding_different_images(self, pipeline_name):
        """
        target: test embedding for normal case
        method:  embedding with different images
        expected: return not identical embeddings
        """
        nums = 2
        embeddings = []
        embedding_pipeline = pipeline(pipeline_name)
        for i in range(nums):
            img = cf.create_image()
            embedding = embedding_pipeline(img)
            embeddings.append(embedding[0][0])
            assert embedding[0][0].size == embedding_size

        assert embeddings[0][0] is not embeddings[1][0]

        return True

    def test_embedding_one_image_multiple_times(self, pipeline_name):
        """
        target: test embedding for normal case
        method: embedding one image for multiple times
        expected: return identical embeddings
        """
        nums = 10
        img = cf.create_image()
        embedding_pipeline = pipeline(pipeline_name)
        embedding_last = embedding_pipeline(img)
        for _ in range(nums):
            embedding = embedding_pipeline(img)
            assert embedding[0][0].size == embedding_size
            assert embedding[0][0].all() == embedding_last[0][0].all()

        return True

    def test_embedding_images_multiple_times(self, pipeline_name):
        """
        target: test embedding for normal case
        method:  embedding images for multiple times
        expected: return embeddings
        """
        nums = 2
        times = 10
        embedding_pipeline = pipeline(pipeline_name)
        for _ in range(times):
            embeddings = []
            for i in range(nums):
                img = cf.create_image()
                embedding = embedding_pipeline(img)
                embeddings.append(embedding)
                assert embedding[0][0].size == embedding_size
            assert embeddings[0][0] is not embeddings[1][0]

        return True

    def test_embedding_concurrent_multi_threads(self, pipeline_name):
        """
        target: test embedding for normal case
        method:  embedding with concurrent multi-processes
        expected: return embeddings
        """
        threads_num = 10
        threads = []
        img = cf.create_image()

        def embedding():
            embedding_pipeline = pipeline(pipeline_name)
            embedding = embedding_pipeline(img)
            assert embedding[0][0].size == embedding_size

        for i in range(threads_num):
            t = threading.Thread(target=embedding)
            threads.append(t)
            t.start()
            time.sleep(0.2)
        for t in threads:
            t.join()

        return True





# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from towhee.dag.graph_repr import GraphRepr
from towhee.dag.variable_repr import VariableRepr
from towhee.dag.dataframe_repr import DataFrameRepr
import unittest

from towhee.dataframe import Variable
from towhee.engine.graph_context import GraphContext
from towhee.engine.operator_context import OpStatus
from towhee.dag import OperatorRepr
from towhee.engine.thread_pool_task_executor import ThreadPoolTaskExecutor

from towhee.tests import CACHE_PATH


class TestGraphCtx(unittest.TestCase):
    """
    Graph ctx test
    """

    def setUp(self):
        self._task_exec = ThreadPoolTaskExecutor('Graph_ctx_test',
                                                 CACHE_PATH)
        self._task_exec.start()

    def tearDown(self):
        if self._task_exec.is_alive():
            self._task_exec.stop()
            self._task_exec.join()

    def _crate_graph(self):

        start_op_repr = OperatorRepr(
            name='_start_op',
            function='_start_op',
            init_args={},
            inputs=[
                {'name': 'num', 'df': '_start_df', 'col': 0}
            ],
            outputs=[{'df': 'op_test_in'}],
            iter_info={'type': 'map'}
        )

        add_op_repr = OperatorRepr(
            name='local/add_operator',
            function='local/add_operator',
            init_args={'factor': 2},
            inputs=[
                {'name': 'num', 'df': 'op_test_in', 'col': 0}
            ],
            outputs=[{'df': 'op_test_out'}],
            iter_info={'type': 'map'}
        )

        end_op_repr = OperatorRepr(
            name='_end_op',
            function='_end_op',
            init_args={},
            inputs=[
                {'name': 'sum', 'df': 'op_test_out', 'col': 0}
            ],
            outputs=[{'df': '_end_df'}],
            iter_info={'type': 'map'}
        )

        df_start_repr = DataFrameRepr(
            name='_start_df',
            columns=[VariableRepr('num', 'int')]
        )

        df_in_repr = DataFrameRepr(
            name='op_test_in',
            columns=[VariableRepr('num', 'int')]
        )

        df_out_repr = DataFrameRepr(
            name='op_test_out',
            columns=[VariableRepr('sum', 'int')]
        )

        df_end_repr = DataFrameRepr(
            name='_end_df',
            columns=[VariableRepr('sum', 'int')]
        )

        op_reprs = {
            start_op_repr.name: start_op_repr,
            add_op_repr.name: add_op_repr,
            end_op_repr.name: end_op_repr
        }

        df_reprs = {
            df_start_repr.name: df_start_repr,
            df_in_repr.name: df_in_repr,
            df_out_repr.name: df_out_repr,
            df_end_repr.name: df_end_repr
        }

        graph_repr = GraphRepr('add', op_reprs, df_reprs)
        graph_ctx = GraphContext(0, graph_repr)
        return graph_ctx
        # df_in = DataFrame(
        #     'op_test_in', {'sum': {'index': 0, 'type': 'int'}})
        # df_in.put((Variable('int', 1), ))
        # df_in.seal()

    def test_graph_ctx(self):
        graph_ctx = self._crate_graph()

        for op in graph_ctx.op_ctxs.values():
            op.start(self._task_exec)

        graph_ctx((Variable('int', 3), ))

        graph_ctx.join()

        for op in graph_ctx.op_ctxs.values():
            self.assertEqual(op.status, OpStatus.FINISHED)

        it = graph_ctx.outputs.map_iter(True)
        for data in it:
            self.assertEqual(data[0].value, 5)

    def test_graph_ctx_failed(self):
        graph_ctx = self._crate_graph()

        for op in graph_ctx.op_ctxs.values():
            op.start(self._task_exec)

        # Error input data
        graph_ctx((Variable('string', 'x'), ))

        graph_ctx.join()

        for name, op in graph_ctx.op_ctxs.items():
            if name not in ['_start_op', '_end_op']:
                self.assertEqual(op.status, OpStatus.FAILED)

        self.assertEqual(graph_ctx.outputs.size, 0)



# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import unittest
from pathlib import Path
from typing import Dict
from queue import Queue
import time

from towhee.engine.operator_runner.map_runner import MapRunner
from towhee.engine.operator_runner.runner_base import RunnerStatus
from towhee.engine.thread_pool_task_executor import ThreadPoolTaskExecutor
from towhee.engine.operator_io._mock_reader import MockReader
from towhee.hub.file_manager import FileManagerConfig, FileManager
from towhee.tests import CACHE_PATH


DATA_QUEUE = Queue()


class MockWriter:
    def __init__(self):
        self.res = []

    def write(self, data: Dict):
        self.res.append(data)


class TestThreadPoolTaskExecutor(unittest.TestCase):
    """
    Thread pool task executor test
    """

    @classmethod
    def setUpClass(cls):
        new_cache = (CACHE_PATH/'test_cache')
        pipeline_cache = (CACHE_PATH/'test_util')
        operator_cache = (CACHE_PATH/'mock_operators')
        fmc = FileManagerConfig()
        fmc.update_default_cache(new_cache)
        pipelines = list(pipeline_cache.rglob('*.yaml'))
        operators = [f for f in operator_cache.iterdir() if f.is_dir()]
        fmc.cache_local_pipeline(pipelines)
        fmc.cache_local_operator(operators)
        FileManager(fmc)

    def setUp(self):
        cache_path = Path(__file__).parent.parent.resolve()
        self._task_exec = ThreadPoolTaskExecutor('tread_pool_task_executor_test',
                                                 cache_path)
        self._task_exec.start()

    def tearDown(self):
        if self._task_exec.is_alive():
            self._task_exec.stop()
            self._task_exec.join()

    def test_pool_with_map_runner(self):
        data_queue = Queue()
        writer = MockWriter()
        hub_op_id = 'local/add_operator'
        runner = MapRunner('test', 0, 'add_operator',
                           hub_op_id, {'factor': 1},
                           MockReader(data_queue), writer)
        self._task_exec.push_task(runner)

        data_queue.put({'num': 1})
        data_queue.put({'num': 2})
        data_queue.put({'num': 3})

        time.sleep(0.1)
        runner.set_stop()
        time.sleep(0.1)
        self._task_exec.stop()

        res = 2
        for item in writer.res:
            self.assertEqual(item[0], res)
            res += 1

        self.assertEqual(runner.status, RunnerStatus.FINISHED)

    def test_pool_with_map_runner_error(self):
        data_queue = Queue()
        writer = MockWriter()
        hub_op_id = 'local/add_operator'
        runner = MapRunner('test', 0, 'add_operator',
                           hub_op_id, {'factor': 1},
                           MockReader(data_queue), writer)
        self._task_exec.push_task(runner)

        data_queue.put('error')
        time.sleep(0.1)
        runner.set_stop()
        time.sleep(0.1)
        self._task_exec.stop()
        self.assertEqual(len(writer.res), 0)
        self.assertEqual(runner.status, RunnerStatus.FAILED)



# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
import json
from PIL import Image
import torch
from torchvision import transforms
import os

from towhee.models.swin_transformer.configs import build_configs
from towhee.models.swin_transformer.model import SwinTransformer
from towhee.models.utils.pretrained_utils import load_pretrained_weights

class SwinTransformerTest(unittest.TestCase):
    name = 'swin_small_patch4_window7_224'
    pwd = os.getcwd()
    resource_dir = pwd + '/towhee/tests/models/resource/'
    arch, model_cfg = build_configs(name)
    model = SwinTransformer(**arch)

    load_pretrained_weights(model, name, model_cfg)
    img = Image.open(resource_dir + 'img.jpg')
    tfms = transforms.Compose([transforms.Resize([256,256]),
                           transforms.CenterCrop([224,224]),
                           transforms.ToTensor(),
                           transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225]), ])

    img = tfms(img).unsqueeze(0)
    with open(resource_dir + 'labels_map.txt', encoding='utf-8') as handler:
        labels_map = json.load(handler)
    _labels_map = []

    for i in range(1000):
        _labels_map.append(labels_map[str(i)])
    labels_map = _labels_map
    model.eval()
    out = model(img)
    scores = torch.nn.functional.softmax(out,1)
    with torch.no_grad():
        outputs = model(img).squeeze(0)

    def test_swin_transformer(self):
        for idx in torch.topk(self.outputs, k=1).indices.tolist():
            #prob = torch.softmax(self.outputs, -1)[idx].item()
            #label = self.labels_map[idx]
            #p = prob * 100
            self.assertEqual('giant panda, panda, panda bear, coon bear, Ailuropoda melanoleuca', self.labels_map[idx])

if __name__ == '__main__':
    unittest.main()



# coding : UTF-8
import sys
sys.path.append("../../tests")
import time
import threading
from towhee import pipeline

data_path = "images/"

embedding_size = 1000

class TestImageObjEmbeddingInvalid:
    """ Test case of invalid embedding interface """

    def test_embedding_no_image(self, pipeline_name):
        """
        target: test embedding for invalid scenario
        method:  embedding with no image, now issue 154
        expected: raise exception
        """
        embedding_pipeline = pipeline(pipeline_name)
        try:
            embedding = embedding_pipeline()
        except RuntimeError as e:
            assert "Input data is empty" in str(e)

        return True

    def test_embedding_no_obj_image(self, pipeline_name):
        """
        target: test embedding for no object image
        method:  embedding with no object image
        expected: raise exception for empty result
        """
        img = "../../towhee_logo.png"
        embedding_pipeline = pipeline(pipeline_name)
        try:
            embedding = embedding_pipeline(img)
        except RuntimeError as e:
            assert "Unkown error" in str(e)

        return True

class TestImageObjEmbeddingValid:
    """ Test case of valid embedding interface """

    def test_embedding_one_image_single_obj(self, pipeline_name):
        """
        target: test embedding for normal case
        method:  embedding with one image
        expected: return embeddings
        """
        embedding_pipeline = pipeline(pipeline_name)
        embedding = embedding_pipeline(data_path + "towhee_test_image0.jpg")
        assert embedding[0][0].size == embedding_size

        return True

    def test_embedding_one_image_multiple_obj(self, pipeline_name):
        """
        target: test embedding for normal case
        method:  embedding with one image
        expected: return embeddings
        """
        embedding_pipeline = pipeline(pipeline_name)
        embedding = embedding_pipeline(data_path + "towhee_test_image1.jpg")
        assert embedding[0][0].size == embedding_size
        assert embedding[1][0].size == embedding_size
        assert len(embedding) == 2

        return True

    def test_embedding_same_images(self, pipeline_name):
        """
        target: test embedding for normal case
        method:  embedding with same images
        expected: return identical embeddings
        """
        embedding_pipeline = pipeline(pipeline_name)
        embedding = embedding_pipeline(data_path + "towhee_test_image0.jpg")
        embedding1 = embedding_pipeline(data_path + "towhee_test_image2.jpg")
        assert embedding[0][0].size == embedding_size
        assert embedding[0][0].all() == embedding1[0][0].all()

        return True

    def test_embedding_different_images(self, pipeline_name):
        """
        target: test embedding for normal case
        method:  embedding with different images
        expected: return not identical embeddings
        """
        nums = 2
        embeddings = []
        embedding_pipeline = pipeline(pipeline_name)
        for i in range(nums):
            embedding = embedding_pipeline(data_path + "towhee_test_image%i.jpg" % i)
            embeddings.append(embedding)
            assert embedding[0][0].size == embedding_size

        assert embeddings[0][0] is not embeddings[1][0]

        return True

    def test_embedding_one_image_multiple_times(self, pipeline_name):
        """
        target: test embedding for normal case
        method: embedding one image for multiple times
        expected: return identical embeddings
        """
        nums = 10
        embedding_pipeline = pipeline(pipeline_name)
        embedding_last = embedding_pipeline(data_path + "towhee_test_image0.jpg")
        for _ in range(nums):
            embedding = embedding_pipeline(data_path + "towhee_test_image0.jpg")
            assert embedding[0][0].size == embedding_size
        assert embedding[0][0].all() == embedding_last[0][0].all()

        return True

    def test_embedding_images_multiple_times(self, pipeline_name):
        """
        target: test embedding for normal case
        method:  embedding images for multiple times
        expected: return embeddings
        """
        nums = 2
        times = 10
        embedding_pipeline = pipeline(pipeline_name)
        for _ in range(times):
            embeddings = []
            for i in range(nums):
                embedding = embedding_pipeline(data_path + "towhee_test_image0.jpg")
                embeddings.append(embedding)
                assert embedding[0][0].size == embedding_size
            assert embeddings[0][0] is not embeddings[1][0]

        return True

    def test_embedding_concurrent_multi_threads(self, pipeline_name):
        """
        target: test embedding for normal case
        method:  embedding with concurrent multi-processes
        expected: return embeddings
        """
        threads_num = 10
        threads = []

        def embedding():
            embedding_pipeline = pipeline(pipeline_name)
            embedding = embedding_pipeline(data_path + "towhee_test_image0.jpg")
            assert embedding[0][0].size == embedding_size

        for i in range(threads_num):
            t = threading.Thread(target=embedding)
            threads.append(t)
            t.start()
            time.sleep(0.2)
        for t in threads:
            t.join()

        return True



# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
from typing import List, Tuple

from towhee.dataframe import DataFrame
from towhee.dataframe import Variable
from towhee.engine.engine import Engine, start_engine
from towhee.engine.pipeline import Pipeline
from towhee.hub.file_manager import FileManagerConfig, FileManager

__all__ = ['DEFAULT_PIPELINES', 'pipeline']

DEFAULT_PIPELINES = {
    'image-embedding': 'towhee/image-embedding-resnet50',
    'image-encoding': 'towhee/image-embedding-resnet50',  # TODO: add encoders
    'music-embedding': 'towhee/music-embedding-vggish',
    'music-encoding': 'towhee/music-embedding-clmr',  # TODO: clmr -> encoder
}

_PIPELINE_CACHE_ENV = 'PIPELINE_CACHE'


class _PipelineWrapper:
    """
    A wrapper class around `Pipeline`.

    The class prevents users from having to create `DataFrame` instances by hand.

    Args:
        pipeline (`towhee.Pipeline`):
            Base `Pipeline` instance for which this object will provide a wrapper for.
    """
    def __init__(self, pipeline_: Pipeline):
        self._pipeline = pipeline_

    def __call__(self, *args) -> List[Tuple]:
        """
        Wraps the input arguments around a `Dataframe` for Pipeline.__call__(). For
        example:
        ```
        >>> p = pipeline('some-pipeline')
        >>> result = p(arg0, arg1)
        ```
        """
        if not args:
            raise RuntimeError('Input data is empty')

        vargs = []
        for arg in args:
            vtype = type(arg).__name__
            vargs.append(Variable(vtype, arg))
        vargs = tuple(vargs)

        # Process the data through the pipeline.
        in_df = DataFrame('_in_df')
        in_df.put(vargs)
        out_df = self._pipeline(in_df)

        res = []
        it = out_df.map_iter()
        for data in it:
            # data is Tuple[Variable]
            data_value = []
            for item in data:
                data_value.append(item.value)
            res.append(tuple(data_value))
        return res


def pipeline(task: str, fmc: FileManagerConfig = FileManagerConfig(), branch: str = 'main', force_download: bool = False):
    """
    Entry method which takes either an input task or path to an operator YAML.

    A `Pipeline` object is created (based on said task) and subsequently added to the
    existing `Engine`.

    Args:
        task (`str`):
            Task name or YAML file location to use.
        fmc (`FileManagerConfig`):
            Optional file manager config for the local instance, defaults to local cache.
        branch (`str`):
            Which branch to use for operators/pipelines on hub, defaults to `main`.
        force_download (`bool`):
            Whether to redownload pipeline and operators.

    Returns
        (`typing.Any`)
            The `Pipeline` output.
    """
    start_engine()

    if os.path.isfile(task):
        yaml_path = task
    else:
        fm = FileManager(fmc)
        task = DEFAULT_PIPELINES.get(task, task)
        yaml_path = fm.get_pipeline(task, branch, force_download)

    engine = Engine()
    pipeline_ = Pipeline(str(yaml_path))
    engine.add_pipeline(pipeline_)

    return _PipelineWrapper(pipeline_)



# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""
Pytorch impletation of 'An Image is Worth 16x16 Words: Transformers for Image Recognition at Scale'.
Jax version: https://github.com/google-research/vision_transformer
"""

import numpy as np
import torch
from torch.utils import model_zoo
from scipy.ndimage import zoom


from towhee.trainer.models.vit.vit_pretrained import PRETRAINED_MODELS


def load_pretrained_weights(
    model,
    model_name=None,
    weights_path=None,
    load_first_conv=True,
    load_fc=True,
    load_repr_layer=False,
    resize_positional_embedding=False,
    verbose=True,
    strict=True,
):
    """
    Load pretrained weights from path or url.
    Args:
        model(nn.Module):
            Model.
        model_name(str):
            Model name
        weights_path(str, optional):
            Path to pretrained weights file on the local disk.
        load_first_conv(bool):
            If patch embedding is loaded.
        load_fc(bool):
            If pretrained weights of fc layer is loaded at the end of the model.
        load_repr_layer(bool)
            If representation is loaded.
        resize_positional_embedding(bool):
            If positional embedding is resized.
        verbose (bool):
            If printing is done when downloading is over.
    """
    print('model_name is '+str(model_name))
    print('weights_path is '+str(weights_path))
    assert bool(model_name), 'Expected model_name'

    # Load or download weights
    if weights_path is None:
        url = PRETRAINED_MODELS[model_name]['url']
        if url:
            print('Please check hub connection in case weights can not be downloaded!')
            state_dict = model_zoo.load_url(url)
        else:
            raise ValueError(f'Pretrained model for {model_name} is not available.')
    else:
        state_dict = torch.load(weights_path)

    # Modifications to load partial state dict
    expected_missing_keys = []
    if not load_first_conv and 'patch_embedding.weight' in state_dict:
        expected_missing_keys += ['patch_embedding.weight', 'patch_embedding.bias']
    if not load_fc and 'fc.weight' in state_dict:
        expected_missing_keys += ['fc.weight', 'fc.bias']
    if not load_repr_layer and 'pre_logits.weight' in state_dict:
        expected_missing_keys += ['pre_logits.weight', 'pre_logits.bias']
    for key in expected_missing_keys:
        state_dict.pop(key)

    # Change size of positional embeddings
    if resize_positional_embedding:
        posemb = state_dict['positional_embedding.pos_embedding']
        posemb_new = model.state_dict()['positional_embedding.pos_embedding']
        state_dict['positional_embedding.pos_embedding'] = \
            resize_positional_embedding_(posemb=posemb, posemb_new=posemb_new,
                has_class_token=hasattr(model, 'class_token'))
        maybe_print(f'Resized positional embeddings from {posemb.shape} to {posemb_new.shape}', verbose)

    # Load state dict
    ret = model.load_state_dict(state_dict, strict=False)
    print(f'ret is {ret}')
    if strict:
        assert set(ret.missing_keys) == set(expected_missing_keys), \
            f'Missing keys when loading pretrained weights: {ret.missing_keys}'
        assert not ret.unexpected_keys, \
            f'Missing keys when loading pretrained weights: {ret.unexpected_keys}'
        maybe_print('Loaded pretrained weights.', verbose)
    else:
        maybe_print(f'Missing keys when loading pretrained weights: {ret.missing_keys}', verbose)
        maybe_print(f'Unexpected keys when loading pretrained weights: {ret.unexpected_keys}', verbose)
        return ret


def maybe_print(s: str, flag: bool):
    if flag:
        print(s)


def as_tuple(x):
    return x if isinstance(x, tuple) else (x, x)


def resize_positional_embedding_(posemb, posemb_new, has_class_token=True):
    """
    Rescale the grid of position embeddings in a sensible manner
    """

    # Deal with class token
    ntok_new = posemb_new.shape[1]
    if has_class_token:  # this means classifier == 'token'
        posemb_tok, posemb_grid = posemb[:, :1], posemb[0, 1:]
        ntok_new -= 1
    else:
        posemb_tok, posemb_grid = posemb[:, :0], posemb[0]

    # Get old and new grid sizes
    gs_old = int(np.sqrt(len(posemb_grid)))
    gs_new = int(np.sqrt(ntok_new))
    posemb_grid = posemb_grid.reshape(gs_old, gs_old, -1)

    # Rescale grid
    zoom_factor = (gs_new / gs_old, gs_new / gs_old, 1)
    posemb_grid = zoom(posemb_grid, zoom_factor, order=1)
    posemb_grid = posemb_grid.reshape(1, gs_new * gs_new, -1)
    posemb_grid = torch.from_numpy(posemb_grid)

    # Deal with class token and return
    posemb = torch.cat([posemb_tok, posemb_grid], dim=1)
    return posemb



# coding : UTF-8

from PIL import Image


def create_image(mode="RGB", size=(200, 200), color=(155, 155, 155)):
    """
    target: create image with specified properties
    method:  use PIL image module
    expected: return image obj
    """
    img = Image.new(mode, size, color)

    return img




# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import unittest
import torch
from towhee.models.layers.cond_conv2d import CondConv2d

class CondConv2dTest(unittest.TestCase):
    def test_cond_conv2d(self):
        in_channels = 32
        out_channels = 64
        kernel_size = 3
        stride = 1
        padding = 'same'
        num_experts = 5

        module = CondConv2d(in_channels, out_channels, kernel_size, stride, padding, num_experts = num_experts)
        routing_weights = torch.FloatTensor([0.2, 0.2, 0.2, 0.2, 0.2]).reshape((1,5))
        x = torch.FloatTensor(1,32,224,224)
        output = module(x, routing_weights)
        gt_output_shape = torch.Size((1,64,224,224))
        self.assertTrue(output.shape == gt_output_shape)




# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import NamedTuple, Union

import numpy as np
from PIL import Image
import torch
from torchvision import transforms

from towhee.operator import Operator


class PytorchTransformOperator(Operator):
    """Uses PyTorch to transform an image (resize, crop, normalize, etc...)

    Args:
        size: (`int`)
            Image size to use. A resize to `size x size` followed by center crop and
            image normalization will be done.
    """

    def __init__(self, size: int) -> None:
        super().__init__()
        # user defined transform
        self.tfms = transforms.Compose([transforms.Resize(size),
                                        transforms.CenterCrop(224),
                                        transforms.ToTensor(),
                                        transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225]),
                                        ])

    def __call__(self, img_tensor: Union[np.ndarray, Image.Image, torch.Tensor]) -> NamedTuple('Outputs', [('img_transformed', torch.Tensor)]):
        if isinstance(img_tensor, Image.Image):
            img_tensor = img_tensor.convert('RGB')
        Outputs = NamedTuple('Outputs', [('img_transformed', torch.Tensor)])
        return Outputs(self.tfms(img_tensor).unsqueeze(0))



# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from typing import Generator, NamedTuple
from towhee.operator import Operator, SharedType


Outputs = NamedTuple("Outputs", [("sum", int)])


class GeneratoOperator(Operator):
    """
    Generator operator.
    """

    def __init__(self) -> None:  # pylint: disable=super-init-not-called
        pass

    def __call__(self, num: int) -> Generator:
        self._num = 0
        while True:
            if self._num >= num:
                break
            else:
                yield Outputs(self._num)
                self._num += 1

    @property
    def shared_type(self):
        return SharedType.NotShareable



# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from towhee.operator import PyOperator
from towhee.utils.log import engine_log


class TestPyOperator(PyOperator):
    """
    A test PyOperator with no functionality.
    """
    def __init__(self):
        super().__init__()
        pass

    def __call__(self):
        engine_log.info('I\'m an NNOperator.')



# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import unittest
import torch
import torchvision

from pathlib import Path

from PIL import Image

from towhee.tests.mock_operators import PYTORCH_OBJECT_DETECTION_OPERATOR_PATH, load_local_operator

cache_path = Path(__file__).parent.parent.resolve()
test_image = cache_path.joinpath('data/dataset/kaggle_dataset_small/train/1cef8fb87d5ca2b8a833b5f9549634aa.jpg')

model = torchvision.models.detection.fasterrcnn_resnet50_fpn(pretrained=True)
device = torch.device('cuda') if torch.cuda.is_available() else torch.device('cpu')


class TestOperator(unittest.TestCase):
    """
    Simple operator test
    """
    def setUp(self) -> None:
        img_pil = Image.open(test_image).convert('RGB')
        self.img_tensor = torchvision.transforms.ToTensor()(img_pil)
        self.images = [self.img_tensor.to(device)]
        self.model_name = 'fasterrcnn_resnet50_fpn'

    def test_func_operator(self):
        test_op = load_local_operator(
            'pytorch_object_detection_operator', PYTORCH_OBJECT_DETECTION_OPERATOR_PATH)
        op = test_op.PyTorchObjectDetectionOperator(self.model_name)
        output = op(self.images)
        self.assertEqual(4, output[0].shape[1])


if __name__ == '__main__':
    unittest.main()



# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from typing import NamedTuple
from collections import defaultdict
from towhee.operator import Operator


class ConcatOperator(Operator):
    """
    Stateful concat operator that can perform scalar and nonscalar concat. If no new data
    is presented, the row will be skipped.
    """

    def __init__(self, repeat_end: bool) -> None:
        super().__init__()
        self._scalar = repeat_end
        self._last_seens = defaultdict(lambda: None)

    def __call__(self, **kwargs):
        ret = []
        for col, val in kwargs.items():
            if val is None:
                if self._scalar:
                    ret.append(self._last_seens[col])
                else:
                    ret.append(None)
            else:
                ret.append(val)
                self._last_seens[col] = val

        #If no new data added, dont return anything.
        if all(val is None for val in kwargs.values()):
            pass
        else:
            cols = [(str(col), type(val)) for col, val in kwargs.items()]
            output = NamedTuple("Outputs", cols)
            return output(*ret) #pylint: disable=not-callable



# coding=utf-8
# Copyright 2020-present the HuggingFace Inc. team and 2021 Zilliz.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
import cv2

from pathlib import Path
from towhee.data.random_erase import RandomErasing

cache_path = Path(__file__).parent.parent.resolve()
test_image = cache_path.joinpath('data/dataset/kaggle_dataset_small/train/001cdf01b096e06d78e9e5112d419397.jpg')


class RandomEraseTest(unittest.TestCase):
    def test_random_erase(self):
        img = cv2.imread(str(test_image))
        RE = RandomErasing(p=1)
        # for i in range(20):
        #     img1 = RE(img.copy())
        #     cv2.imshow("test", img1)
        #     cv2.waitKey(1000)
        img1 = RE(img.copy())
        self.assertEqual(img.shape, img1.shape)


if __name__ == '__main__':
    unittest.main(verbosity=1)
